@@ -16,7 +16,11 @@ def countReadsInRegions_wrapper(args):
 
 def countReadsInRegions_worker(chrom, start, end, bamFilesList,
                                stepSize, binLength, defaultFragmentLength,
-                               skipZeros=False, bedRegions=None):
+                               skipZeros=False,
+                               extendPairedEnds=True,
+                               minMappingQuality=None,
+                               ignoreDuplicates=False,
+                               bedRegions=None):
     """ counts the reads in each bam file at each 'stepSize' position
     within the interval start, end for a 'binLength' window.
     Because the idea is to get counts for window positions at
@@ -48,6 +52,18 @@ def countReadsInRegions_worker(chrom, start, end, bamFilesList,
     array([[ 2.],
            [ 4.]])
 
+    Test min mapping quality
+    >>> np.transpose(countReadsInRegions_worker(test.chrom, 0, 200,
+    ... [test.bamFile1, test.bamFile2], 50, 25, 0, minMappingQuality=40))
+    array([[ 0.,  0.,  0.,  1.],
+           [ 0.,  0.,  0.,  1.]])
+
+    Test ignore duplicates
+    >>> np.transpose(countReadsInRegions_worker(test.chrom, 0, 200,
+    ... [test.bamFile1, test.bamFile2], 50, 25, 0, ignoreDuplicates=True))
+    array([[ 0.,  0.,  1.,  1.],
+           [ 0.,  1.,  1.,  1.]])
+
     Test bed regions:
     >>> bedRegions = [(test.chrom, 10, 20), (test.chrom, 150, 160)]
     >>> np.transpose(countReadsInRegions_worker(test.chrom, 0, 200,
@@ -83,14 +99,17 @@ def countReadsInRegions_worker(chrom, start, end, bamFilesList,
 
     for chrom, start, end, binLength in regionsToConsider:
         avgReadsArray = []
-        for bam in bamHandlers:
-            avgReadsArray.append(
+        for bam in bamHandlers: 
+           avgReadsArray.append(
                 getCoverageOfRegion(bam,
                                     chrom, start, end,
                                     binLength,
                                     defaultFragmentLength,
                                     extendPairedEnds,
-                                    zerosToNans )[0])
+                                    zerosToNans,
+                                    minMappingQuality=minMappingQuality,
+                                    ignoreDuplicates=ignoreDuplicates,
+                                    )[0])
         # skip if any of the bam files returns a NaN
         if np.isnan(sum(avgReadsArray)):
             continue
@@ -105,7 +124,7 @@ def countReadsInRegions_worker(chrom, start, end, bamFilesList,
         print "%s countReadsInRegions_worker: processing %d " \
             "(%.1f per sec) @ %s:%s-%s"  % \
             (multiprocessing.current_process().name,
-             rows, rows / (endTime - startTime), chrom, start, end )
+             rows, rows / (endTime - startTime), chrom, start, end)
 
     return np.array(subNum_reads_per_bin).reshape(rows, len(bamFilesList))
 
@@ -113,8 +132,11 @@ def countReadsInRegions_worker(chrom, start, end, bamFilesList,
 def getNumReadsPerBin(bamFilesList, binLength, numberOfSamples,
                       defaultFragmentLength, numberOfProcessors=1,
                       skipZeros=True, verbose=False, region=None,
-                      bedFile=None,
-                      chrsToSkip=[]):
+                      bedFile=None, extendPairedEnds=True,
+                      minMappingQuality=None,
+                      ignoreDuplicates=False,
+                      chrsToSkip=[],
+                      stepSize=None):
 
     r"""
     This function visits a number of sites and returs a matrix containing read
@@ -160,14 +182,16 @@ def getNumReadsPerBin(bamFilesList, binLength, numberOfSamples,
     chrNames, chrLengths = zip(*chromSizes)
 
     genomeSize = sum(chrLengths)
-    max_mapped = max( [ x.mapped for x in  bamFilesHandlers ] )
+    max_mapped = max([ x.mapped for x in  bamFilesHandlers ])
 
     reads_per_bp = float(max_mapped) / genomeSize
-#    chunkSize =  int(100 / ( reads_per_bp  * len(bamFilesList)) )
+    #chunkSize =  int(100 / (reads_per_bp  * len(bamFilesList)) )
 
-    stepSize = max(int( float(genomeSize) / numberOfSamples ), 1 )
+    # if stepSize is given, this overrides the numberOfSamples
+    if stepSize is None:
+        stepSize = max(int(float(genomeSize) / numberOfSamples), 1)
 
-    chunkSize =  int (stepSize * 1e3 / ( reads_per_bp  * len(bamFilesHandlers)) )
+    chunkSize =  int (stepSize * 1e3 / (reads_per_bp  * len(bamFilesHandlers)))
     [ bam_h.close() for bam_h in bamFilesHandlers]
 
     if verbose:
@@ -177,8 +201,10 @@ def getNumReadsPerBin(bamFilesList, binLength, numberOfSamples,
         # in case a region is used, append the tilesize
         region += ":{}".format(binLength)
 
-    imap_res = mapReduce.mapReduce( (bamFilesList, stepSize, binLength,
-                                     defaultFragmentLength, skipZeros),
+    imap_res = mapReduce.mapReduce((bamFilesList, stepSize, binLength,
+                                     defaultFragmentLength, skipZeros,
+                                     extendPairedEnds, minMappingQuality,
+                                     ignoreDuplicates),
                                     countReadsInRegions_wrapper,
                                     chromSizes,
                                     genomeChunkLength=chunkSize,
@@ -318,18 +344,22 @@ def getCoverageOfRegion(bamHandle, chrom, start, end, tileSize,
     if length % tileSize > 0:
         newLength = length - (length % tileSize)
         if debug:
-            print  "length of region ({}) is not a multiple of tileSize {}\nThe region is being chopped to length {} bp".format(length, tileSize, newLength)
+            print  "length of region ({}) is not a multiple of "\
+                "tileSize {}\nThe region is being chopped to length "\
+                "{} bp".format(length, tileSize, newLength)
 
-    vectorLength = length/tileSize
-    coverage   = np.zeros( vectorLength, dtype='float64' )
+    vectorLength = length / tileSize
+    coverage = np.zeros(vectorLength, dtype='float64')
 
     startTime = time.time()
-    #caching seems faster. TODO: profile the function
+    # caching seems faster. TODO: profile the function
     c = 0
     if chrom in bamHandle.references:
-        reads = [ r for r in bamHandle.fetch( chrom, start, end )]
+        # r.flag & 4 == 0 is to skip unmapped reads
+        reads = [r for r in bamHandle.fetch(chrom, start, end)
+                 if r.flag & 4 == 0]
     else:
-        raise NameError( "chromosome {} not found in bam file".format(chrom) )
+        raise NameError("chromosome {} not found in bam file".format(chrom))
 
     prev_start_pos = None # to store the start positions
                           # of previous processed read pair
@@ -338,22 +368,24 @@ def getCoverageOfRegion(bamHandle, chrom, start, end, tileSize,
         if minMappingQuality and read.mapq < minMappingQuality:
             continue
 
-        # get rid of duplicate reads that have same position on each of the 
+        # get rid of duplicate reads that have same position on each of the
         # pairs
         if ignoreDuplicates and prev_start_pos  \
                 and prev_start_pos == (read.pos, read.pnext, read.is_reverse):
             continue
 
         try:
-            fragmentStart, fragmentEnd = fragmentFromRead_func(read, defaultFragmentLength, 
-                                                               extendPairedEnds,
-                                                               maxPairedFragmentLength=maxPairedFragmentLength)
+            fragmentStart, fragmentEnd = \
+                fragmentFromRead_func(read, defaultFragmentLength,
+                                      extendPairedEnds,
+                                      maxPairedFragmentLength=maxPairedFragmentLength)
+
             fragmentLength = fragmentEnd - fragmentStart
             if fragmentLength == 0:
                 fragmentLength = defaultFragmentLength
 
-            vectorStart = max( (fragmentStart - start)/tileSize, 0)
-            vectorEnd   = min( np.ceil(float(fragmentEnd   - start)/tileSize).astype('int'), 
+            vectorStart = max((fragmentStart - start)/tileSize, 0)
+            vectorEnd   = min(np.ceil(float(fragmentEnd   - start)/tileSize).astype('int'), 
                                vectorLength)
 
             coverage[vectorStart:vectorEnd] +=  1
@@ -420,12 +452,12 @@ def getSmoothRange(tileIndex, tileSize, smoothRange, maxPosition):
     smoothTilesLeft = int(np.ceil(smoothTilesSide))
     smoothTilesRight = int(np.floor(smoothTilesSide)) + 1
 
-    indexStart = max( tileIndex - smoothTilesLeft, 0 )
-    indexEnd   = min( maxPosition, tileIndex + smoothTilesRight )
+    indexStart = max(tileIndex - smoothTilesLeft, 0)
+    indexEnd   = min(maxPosition, tileIndex + smoothTilesRight)
     return (indexStart, indexEnd)
 
 class Tester():
-    def __init__( self ):
+    def __init__(self):
         """
         The distribution of reads between the two bam files is as follows.
 
@@ -449,18 +481,18 @@ class Tester():
         global debug
         debug = 0
 
-    def getRead( self, readType ):
+    def getRead(self, readType):
         """ prepare arguments for test
         """
         bam = bamHandler.openBam(self.bamFile_PE)
         if readType == 'paired-reverse':
-            read = [x for x in bam.fetch('chr2', 5000081,5000082 )][0]
+            read = [x for x in bam.fetch('chr2', 5000081,5000082)][0]
         elif readType == 'single-forward':
-            read = [x for x in bam.fetch('chr2', 5001491, 5001492 )][0]
+            read = [x for x in bam.fetch('chr2', 5001491, 5001492)][0]
         elif readType == 'single-reverse':
-            read = [x for x in bam.fetch('chr2', 5001700, 5001701 )][0]
+            read = [x for x in bam.fetch('chr2', 5001700, 5001701)][0]
         else: # by default a forward paired read is returned
-            read = [x for x in bam.fetch('chr2', 5000027,5000028 )][0]
+            read = [x for x in bam.fetch('chr2', 5000027,5000028)][0]
         return read
 
 
